library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity Hamming is
    Port ( clk : in std_logic;
           din : in std_logic_vector(23 downto 0);
           cc_ecc : in std_logic;
           dout : out std_logic_vector(19 downto 0);
           cc_rxena : in std_logic);
end Hamming;

architecture my_arch of Hamming is
component DEC_FUNC
    port(        
     clk			: in STD_LOGIC;
          -- Data Inputs 
	  b10in		: in STD_LOGIC_vector(9 downto 0);			
	  -- Output data terms
	  b8out		: out STD_LOGIC_vector(7 downto 0);
	  kout		: out STD_LOGIC		-- Asserted when special character
								-- is detected in transmission
          );
end component;
component err_chk
    port(        
     clk			: in STD_LOGIC;
          -- Data Inputs 
	  b10in		: in STD_LOGIC_vector(9 downto 0);			
	  err_out	: out STD_LOGIC			-- Asserted when error has been detected
          );
end component;
signal din_q: std_logic_vector(23 downto 0) := (others => '0');
signal d: std_logic_vector(24 downto 1) := (others => '0');
signal q: std_logic_vector(19 downto 0) := (others => '0');
signal q_l: std_logic_vector(19 downto 0) := (others => '0');
signal cp: std_logic_vector(5 downto 0) := (others => '0');
signal c: std_logic_vector(6 downto 0) := (others => '0');
signal cntr: std_logic_vector(7 downto 0) := (others => '0');
signal h_word: std_logic := '0';
signal en_correct: std_logic_vector(3 downto 0) := (others => '0');
signal sgl_errp : std_logic;
signal dbl_errp : std_logic;
signal locked : std_logic;
signal d10b: std_logic_vector(19 downto 0) := (others => '0');
signal d8b: std_logic_vector(15 downto 0) := (others => '0');
signal d8b_dl: std_logic_vector(15 downto 0) := (others => '0');
signal d8b_dl2: std_logic_vector(15 downto 0) := (others => '0');
signal s: std_logic_vector(1 downto 0) := (others => '0');
signal k: std_logic_vector(1 downto 0) := (others => '0');
signal k_dl: std_logic_vector(1 downto 0) := (others => '0');
signal k_dl2: std_logic_vector(1 downto 0) := (others => '0');
signal par_err: std_logic := '0';
signal even_par: std_logic := '0';
signal pre_err: std_logic_vector(3 downto 0) := (others => '0');
signal err: std_logic_vector(3 downto 0) := (others => '0');
signal err_dl: std_logic_vector(2 downto 0) := (others => '0');
signal err_dl2: std_logic_vector(2 downto 0) := (others => '0');
signal sgl_err : std_logic;
signal dbl_err : std_logic;
signal header : std_logic;
signal trailer : std_logic;
signal idle_next : std_logic;
signal k28 : std_logic_vector(1 downto 0);
begin
process(clk)
begin
  if(clk'event and clk = '1')then
    din_q <= din;
    d <= din_q;
	 cp(0) <= d(21) xor d(1) xor d(2) xor d(4) xor d(5) xor d(7) xor
	         d(9) xor d(11) xor d(12) xor d(14) xor d(16) xor d(18) xor d(20);
	 cp(1) <= d(22) xor d(1) xor d(3) xor d(4) xor d(6) xor d(7) xor
	         d(10) xor d(11) xor d(13) xor d(14) xor d(17) xor d(18);
	 cp(2) <= d(23) xor d(2) xor d(3) xor d(4) xor d(8) xor d(9) xor
	         d(10) xor d(11) xor d(15) xor d(16) xor d(17) xor d(18);
	 cp(3) <= d(24) xor d(5) xor d(6) xor d(7) xor d(8) xor d(9) xor
	         d(10) xor d(11) xor d(19) xor d(20);
	 cp(4) <= d(12) xor d(13) xor d(14) xor d(15) xor d(16) xor d(17) xor
	         d(18) xor d(19) xor d(20);
	 cp(5) <= d(3) xor d(6) xor d(8) xor d(10) xor d(13) xor d(15) xor
	         d(17) xor d(19) xor d(22) xor d(23) xor d(24);
-----------------------------------------------------------------------------------
	 c(0) <= cp(0) xor d(2) xor d(4) xor d(6) xor d(7) xor d(9) xor d(11) xor
	         d(13) xor d(15) xor d(17) xor d(19);
	 c(1) <= cp(1) xor d(1) xor d(2) xor d(5) xor d(6) xor d(8) xor d(9) xor
	         d(12) xor d(13) xor d(16) xor d(17) xor d(20);
	 c(2) <= cp(2) xor d(3) xor d(4) xor d(5) xor d(6) xor d(10) xor d(11) xor
	         d(12) xor d(13) xor d(18) xor d(19) xor d(20);
	 c(3) <= cp(3) xor d(1) xor d(2) xor d(3) xor d(4) xor d(5) xor d(6) xor
	         d(14) xor d(15) xor d(16) xor d(17) xor d(18) xor d(19) xor d(20);
	 c(4) <= cp(4) xor d(1) xor d(2) xor d(3) xor d(4) xor d(5) xor d(6) xor d(21);
	 c(5) <= not d(6) xor d(7) xor d(8) xor d(9) xor d(10) xor d(11) xor d(12) xor d(13) xor d(14) xor d(15) xor
	         d(16) xor d(17) xor d(18) xor d(19) xor d(20) xor d(22);
	 c(6) <= not cp(5) xor d(1) xor d(3) xor d(5) xor d(8) xor d(10) xor d(12) xor
	         d(14) xor d(16) xor d(18) xor d(20) xor d(21) xor d(22) xor d(23);
	 q <= d(20 downto 1);
	 if(cntr /= "00000000")then
	   h_word <= not h_word;
	 end if;
	 if(cntr = "00000000" or dbl_err = '1')then
	   cntr <= cntr - 1;
	 elsif(cntr /= "11111111" and sgl_err = '0' and h_word = '0')then
	   cntr <= cntr + 1;
	 end if;
	 pre_err(3 downto 2) <= pre_err(1 downto 0);
	 if(h_word = '1')then
	   en_correct <= "0000";
	 else
		en_correct(0) <= cc_ecc and not pre_err(3) and not pre_err(2) and not pre_err(1);
		en_correct(1) <= cc_ecc and not pre_err(3) and not pre_err(2) and not pre_err(0);
		en_correct(2) <= cc_ecc and not pre_err(3) and not pre_err(1) and not pre_err(0);
		en_correct(3) <= cc_ecc and not pre_err(2) and not pre_err(1) and not pre_err(0);
	 end if;
	 case pre_err is
	   when "0001" | "0010" | "0100" | "1000" =>
		  sgl_errp <= '1';
		  dbl_errp <= '0';
	   when "0000" =>
		  sgl_errp <= '0';
		  dbl_errp <= '0';
		when others => 
		  sgl_errp <= '0';
		  dbl_errp <= '1';
	 end case;
 end if;
end process;
i_pre_chk_l: err_chk
  port map(
     clk => clk,
	  b10in => din_q(9 downto 0),
	  err_out => pre_err(0)
  );
i_pre_chk_h: err_chk
  port map(
     clk => clk,
	  b10in => din_q(19 downto 10),
	  err_out => pre_err(1)
  );
-- when h_word = '1' q has high word and q_l has low word and c is valid
-- if there are more than one illegal b10b8 byte, do not correct
process(clk,cc_rxena)
begin
  if(clk'event and clk = '1')then
    if(c = "1011010" and en_correct(2) = '1')then
	   q_l(0) <= not q(0);
	 else
	   q_l(0) <= q(0);
	 end if;
    if(c = "0011011" and en_correct(2) = '1')then
	   q_l(1) <= not q(1);
	 else
	   q_l(1) <= q(1);
	 end if;
    if(c = "1011100" and en_correct(2) = '1')then
	   q_l(2) <= not q(2);
	 else
	   q_l(2) <= q(2);
	 end if;
    if(c = "0011101" and en_correct(2) = '1')then
	   q_l(3) <= not q(3);
	 else
	   q_l(3) <= q(3);
	 end if;
    if(c = "1011110" and en_correct(2) = '1')then
	   q_l(4) <= not q(4);
	 else
	   q_l(4) <= q(4);
	 end if;
    if(c = "0011111" and en_correct(2) = '1')then
	   q_l(5) <= not q(5);
	 else
	   q_l(5) <= q(5);
	 end if;
    if(c = "0100001" and en_correct(2) = '1')then
	   q_l(6) <= not q(6);
	 else
	   q_l(6) <= q(6);
	 end if;
    if(c = "1100010" and en_correct(2) = '1')then
	   q_l(7) <= not q(7);
	 else
	   q_l(7) <= q(7);
	 end if;
    if(c = "0100011" and en_correct(2) = '1')then
	   q_l(8) <= not q(8);
	 else
	   q_l(8) <= q(8);
	 end if;
    if(c = "1100100" and en_correct(2) = '1')then
	   q_l(9) <= not q(9);
	 else
	   q_l(9) <= q(9);
	 end if;
    if(c = "0100101" and en_correct(3) = '1')then
	   q_l(10) <= not q(10);
	 else
	   q_l(10) <= q(10);
	 end if;
    if(c = "1100110" and en_correct(3) = '1')then
	   q_l(11) <= not q(11);
	 else
	   q_l(11) <= q(11);
	 end if;
    if(c = "0100111" and en_correct(3) = '1')then
	   q_l(12) <= not q(12);
	 else
	   q_l(12) <= q(12);
	 end if;
    if(c = "1101000" and en_correct(3) = '1')then
	   q_l(13) <= not q(13);
	 else
	   q_l(13) <= q(13);
	 end if;
    if(c = "0101001" and en_correct(3) = '1')then
	   q_l(14) <= not q(14);
	 else
	   q_l(14) <= q(14);
	 end if;
    if(c = "1101010" and en_correct(3) = '1')then
	   q_l(15) <= not q(15);
	 else
	   q_l(15) <= q(15);
	 end if;
    if(c = "0101011" and en_correct(3) = '1')then
	   q_l(16) <= not q(16);
	 else
	   q_l(16) <= q(16);
	 end if;
    if(c = "1101100" and en_correct(3) = '1')then
	   q_l(17) <= not q(17);
	 else
	   q_l(17) <= q(17);
	 end if;
    if(c = "0101101" and en_correct(3) = '1')then
	   q_l(18) <= not q(18);
	 else
	   q_l(18) <= q(18);
	 end if;
    if(c = "1101110" and en_correct(3) = '1')then
	   q_l(19) <= not q(19);
	 else
	   q_l(19) <= q(19);
	 end if;
    if(c = "0000011" and en_correct(0) = '1')then
	   d10B(0) <= not q_l(0);
	 else
	   d10B(0) <= q_l(0);
	 end if;
    if(c = "0000101" and en_correct(0) = '1')then
	   d10B(1) <= not q_l(1);
	 else
	   d10B(1) <= q_l(1);
	 end if;
    if(c = "1000110" and en_correct(0) = '1')then
	   d10B(2) <= not q_l(2);
	 else
	   d10B(2) <= q_l(2);
	 end if;
    if(c = "0000111" and en_correct(0) = '1')then
	   d10B(3) <= not q_l(3);
	 else
	   d10B(3) <= q_l(3);
	 end if;
    if(c = "0001001" and en_correct(0) = '1')then
	   d10B(4) <= not q_l(4);
	 else
	   d10B(4) <= q_l(4);
	 end if;
    if(c = "1001010" and en_correct(0) = '1')then
	   d10B(5) <= not q_l(5);
	 else
	   d10B(5) <= q_l(5);
	 end if;
    if(c = "0001011" and en_correct(0) = '1')then
	   d10B(6) <= not q_l(6);
	 else
	   d10B(6) <= q_l(6);
	 end if;
    if(c = "1001100" and en_correct(0) = '1')then
	   d10B(7) <= not q_l(7);
	 else
	   d10B(7) <= q_l(7);
	 end if;
    if(c = "0001101" and en_correct(0) = '1')then
	   d10B(8) <= not q_l(8);
	 else
	   d10B(8) <= q_l(8);
	 end if;
    if(c = "1001110" and en_correct(0) = '1')then
	   d10B(9) <= not q_l(9);
	 else
	   d10B(9) <= q_l(9);
	 end if;
    if(c = "0001111" and en_correct(1) = '1')then
	   d10B(10) <= not q_l(10);
	 else
	   d10B(10) <= q_l(10);
	 end if;
    if(c = "0010001" and en_correct(1) = '1')then
	   d10B(11) <= not q_l(11);
	 else
	   d10B(11) <= q_l(11);
	 end if;
    if(c = "1010010" and en_correct(1) = '1')then
	   d10B(12) <= not q_l(12);
	 else
	   d10B(12) <= q_l(12);
	 end if;
    if(c = "0010011" and en_correct(1) = '1')then
	   d10B(13) <= not q_l(13);
	 else
	   d10B(13) <= q_l(13);
	 end if;
    if(c = "1010100" and en_correct(1) = '1')then
	   d10B(14) <= not q_l(14);
	 else
	   d10B(14) <= q_l(14);
	 end if;
    if(c = "0010101" and en_correct(1) = '1')then
	   d10B(15) <= not q_l(15);
	 else
	   d10B(15) <= q_l(15);
	 end if;
    if(c = "1010110" and en_correct(1) = '1')then
	   d10B(16) <= not q_l(16);
	 else
	   d10B(16) <= q_l(16);
	 end if;
    if(c = "0010111" and en_correct(1) = '1')then
	   d10B(17) <= not q_l(17);
	 else
	   d10B(17) <= q_l(17);
	 end if;
    if(c = "1011000" and en_correct(1) = '1')then
	   d10B(18) <= not q_l(18);
	 else
	   d10B(18) <= q_l(18);
	 end if;
    if(c = "0011001" and en_correct(1) = '1')then
	   d10B(19) <= not q_l(19);
	 else
	   d10B(19) <= q_l(19);
	 end if;
  end if;
end process;
-- B10/B8 decoding
i_err_chk_l: err_chk
  port map(
     clk => clk,
	  b10in => d10B(9 downto 0),
	  err_out => err(2)
  );
i_err_chk_h: err_chk
  port map(
     clk => clk,
	  b10in => d10B(9 downto 0),
	  err_out => err(3)
  );
i_dec_l: dec_func
  port map(
     clk => clk,
	  b10in => d10B(9 downto 0),
	  b8out => d8B(7 downto 0),
	  kout => k(0)
  );
i_dec_h: dec_func
  port map(
     clk => clk,
	  b10in => d10B(19 downto 10),
	  b8out => d8B(15 downto 8),
	  kout => k(1)
  );
-- generate dout
process(clk,cc_rxena)
begin
  if(cc_rxena = '0')then
    s <= (others => '0');
	 locked <= '0';
  elsif(clk'event and clk = '1')then
-- header has high byte K28.0 or K28.4, idle has both as K28.5
-- data has both D.X.Y and trailer has low byte K28.1
   if(locked = '0')then
	  s <= "00";
	else
	 case s is
	   when "00" =>
		  if(header = '1' or (k_dl2 = "10" and k_dl = "00") or
			 (k_dl = "00" and k = "00"))then
		    s <= "11";
		  end if;
	   when "11" =>
		  s <= "10";
	   when "10" =>
		  if((trailer = '1' and k_dl = "11") or
			 (k_dl = "11" and k(1) = '1') or idle_next = '1')then
		    s <= "01";
		  end if;
	   when others =>
		  s <= "00";
	 end case;
	end if;
   if(cntr(7 downto 6) = "11" and dbl_err = '0' and sgl_err = '0' and h_word = '0')then
	  locked <= '1';
   elsif(cntr(7) = '0')then
	  locked <= '0';
   end if;	
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(c /= "0000000")then
	   par_err <= '1';
	 else
	   par_err <= '0';
	 end if;
	 even_par <= not c(6) xor c(0);
	 if(h_word = '0')then
	   err(0) <= par_err and not even_par and not dbl_errp; -- single bit error
	   err(1) <= (par_err and even_par) or dbl_errp or (not par_err and sgl_errp); -- multiple bit error
	 end if;	
    d8b_dl <= d8b;
    d8b_dl2 <= d8b_dl;
	 dout(15 downto 0) <= d8b_dl2;
	 k_dl <= k;
	 k_dl2 <= k_dl;
	 err_dl <= (err(3) or err(2)) & err(1 downto 0);
	 err_dl2 <= err_dl;
	 if(d8b(4 downto 0) = "11100" and k(0) = '1')then
	   K28(0) <= '1';
	 else
	   K28(0) <= '0';
	 end if;
	 if(d8b(12 downto 8) = "11100" and k(1) = '1')then
	   K28(1) <= '1';
	 else
	   K28(1) <= '0';
	 end if;
	 if(k28 = "01" and d8b_dl(7 downto 5) = "001")then
	   trailer <= '1';
	 else
	   trailer <= '0';
	 end if;
	 if(k28 = "10" and d8b_dl(14 downto 13) = "00")then
	   header <= '1';
	 else
	   header <= '0';
	 end if;
	 if(k28 = "11" and d8b(7 downto 5) = "101")then
	   idle_next <= '1';
	 else
	   idle_next <= '0';
	 end if;
	 if(err_dl2(1) = '1' or (cc_ecc = '1' and (err_dl2(2) = '1' or err_dl(2) = '1')))then
	   dbl_err <= h_word;
	 else
	   dbl_err <= '0';
	 end if;
	 if(err_dl2(0) = '1' and ((err_dl2(2) = '0' and  err_dl(2) = '0') or cc_ecc = '0'))then
	   sgl_err <= h_word;
	 else
	   sgl_err <= '0';
	 end if;
  end if;
end process;
dout(19 downto 16) <= dbl_err & sgl_err & s;
end my_arch;
