-- **************************************************************************
-- Owner:		Xilinx Inc.
-- File:  		dec_func.vhd
--
-- Purpose: 		Main 8B/10B decoder function.  Decodes serial data
--			stream following 5B/6B and 3B/4B encoding schemes.  Decodes
--			incoming 10-bit transmitted serial data stream of ain .. jin
--			to byte wide data AOUT .. HOUT.  Incoming data includes control
--			characters i and j.  Detects transmission of special characters 
--			and asserts KOUT respectively.
--
-- Author:		Jennifer Jenkins
-- Date:		3-31-2000
--
-- **************************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity err_chk is
    port(
        
     clk			: in STD_LOGIC;
          -- Data Inputs (ain .. jin)
	  b10in		: in STD_LOGIC_vector(9 downto 0);			
	  err_out	: out STD_LOGIC			-- Asserted when error has been detected
          );

end err_chk;



architecture BEHAVIOUR of err_chk is
signal ain,bin,cin,din,ein,fin,gin,hin,iin,jin: std_logic;
signal err_cl1, err_cl2, err_cl3 : STD_LOGIC;
begin
ain <= b10in(0);
bin <= b10in(1);
cin <= b10in(2);
din <= b10in(3);
ein <= b10in(4);
iin <= b10in(5);
fin <= b10in(6);
gin <= b10in(7);
hin <= b10in(8);
jin <= b10in(9);
	err_cl1 <= ((ain and bin and cin and din) or
		   (((ain and bin and iin) or (cin and din) or fin or
		     ((ain or bin) and (cin or din))) and
		    (not(not(gin) or not(jin) or not(hin) or ein))));
	err_cl2 <= ((not(cin or din) or not(fin) or
		    not((ain or bin or iin) and ((ain and bin) or (cin and din)))) and
		   (not(not(ein) or gin or hin or jin)));
	err_cl3 <= ((((not(cin and din) and not(ain or bin)) or
		     (not(ain or bin or cin or din)) or
		     (not(fin or gin or hin))) and not(iin) and not(ein)) OR
		   (gin and hin and jin and not(iin) and ein) OR
		   ((not(gin or hin or jin)) and iin and not(ein)) OR
		   (((fin and gin and hin) or
		     (cin and din and (ain or bin)) or
		     (ain and bin and (cin or din))) and iin and ein));

	-- ***************** Process: SEQUENTIAL ************************
    	-- Purpose: 	Decoding control logic.  Synchronize control logic 
	-- 		for decoding data inputs.
    
    	SEQUENTIAL: process (clk)
    	begin
        if clk'event and (clk = '1') then
				err_out <= (not(ain or bin) and not(cin or din)) or 
--					   err_cl1 or err_cl2 or err_cl3;
					   err_cl1 or err_cl2;
        end if;
    	end process SEQUENTIAL;
end BEHAVIOUR;
































