library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity pci is
    port (
-- pci clock and reset signals
        clk33: in STD_LOGIC;
        clk133: in STD_LOGIC;
        nRST: in STD_LOGIC;
-- pci signals
        IDSEL: in STD_LOGIC;
        AD: inout STD_LOGIC_VECTOR (31 downto 0);
        nCBE: in STD_LOGIC_VECTOR (3 downto 0);
        PAR: inout STD_LOGIC;
        nFRAME: in STD_LOGIC;
        nIRDY: in STD_LOGIC;
        nDEVSEL: out STD_LOGIC;
        nTRDY: out STD_LOGIC;
        nSTOP: out STD_LOGIC;
        nPERR: out STD_LOGIC;
        nSERR: out STD_LOGIC;
        gc_automon: out STD_LOGIC;
-- lfifo half full, insert wait state in burst read
        lfifo_high: in STD_LOGIC_VECTOR(2 downto 0);
-- monitoring counter count inputs
-- uncorrectable and correctable link errors
        update: in STD_LOGIC_VECTOR(2 downto 0);
        cerr: in STD_LOGIC_VECTOR(2 downto 0);
        uerr: in STD_LOGIC_VECTOR(2 downto 0);
        BADID: in STD_LOGIC_VECTOR(2 downto 0);
        WCOUNT: in STD_LOGIC_VECTOR(2 downto 0);
-- fifo signals
-- sram data in
        fifo_di: in STD_LOGIC_VECTOR (32 downto 0);
        fifo_we: in STD_LOGIC;
        fifo_wa: in STD_LOGIC_VECTOR (7 downto 0);
        trailer: in STD_LOGIC_VECTOR (2 downto 0);
        header: in STD_LOGIC_VECTOR (2 downto 0);
        CS_EMPTY: in STD_LOGIC_VECTOR (2 downto 0);
        CS_OVER: in STD_LOGIC_VECTOR (2 downto 0);
        CS_FULL: in STD_LOGIC_VECTOR (2 downto 0);
-- fifo_read down counts pfifo_wc
        pci_read: out STD_LOGIC_vector(2 downto 0);
        pci_ch: out STD_LOGIC_VECTOR (1 downto 0);
-- control signal out
        cc_ecc: out STD_LOGIC_VECTOR (2 downto 0);
        rst_out: out STD_LOGIC_VECTOR (2 downto 0);
        gc_block: out STD_LOGIC;
        gc_tstrip: out STD_LOGIC;
        gc_idmatch: out STD_LOGIC;
        gc_efifo: out STD_LOGIC;
        gc_ififo: out STD_LOGIC;
        gc_rfifo: out STD_LOGIC;
        gc_ovrl: out STD_LOGIC_VECTOR (1 downto 0);
        cc_rxena: out STD_LOGIC_VECTOR (2 downto 0);
        data_av: out STD_LOGIC_VECTOR (2 downto 0)
    );
end pci;

architecture my_arch of pci is
COMPONENT lpm_ram_dp
   GENERIC ( LPM_WIDTH: POSITIVE := 33;
             LPM_WIDTHAD: POSITIVE := 8;
             LPM_NUMWORDS: NATURAL := 256;
             LPM_TYPE: STRING := "LPM_RAM_DP";
             LPM_INDATA: STRING := "REGISTERED";
             LPM_OUTDATA: STRING := "REGISTERED";
             LPM_RDADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_WRADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_FILE: STRING := "UNUSED";
             LPM_HINT: STRING := "UNUSED"

           );
   PORT    ( rdaddress, wraddress: IN STD_LOGIC_VECTOR(LPM_WIDTHAD-1 DOWNTO 0);
             rdclock, wrclock: IN STD_LOGIC := '0';
             rden, rdclken, wrclken: IN STD_LOGIC := '1';
             wren: IN STD_LOGIC; 
             data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0);
             q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
COMPONENT lpm_counter
   GENERIC (LPM_WIDTH: POSITIVE := 17;
      LPM_MODULUS: NATURAL := 0;
      LPM_DIRECTION: STRING := "UNUSED";
      LPM_AVALUE: STRING := "UNUSED";
      LPM_SVALUE: STRING := "UNUSED";
      LPM_PVALUE: STRING := "UNUSED";
      LPM_TYPE: STRING := "LPM_COUNTER";
      LPM_HINT: STRING := "UNUSED");
   PORT (data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0) := (OTHERS => '0');
      clock: IN STD_LOGIC;
      cin: IN STD_LOGIC := '0';
      clk_en: IN STD_LOGIC := '1';
      cnt_en: IN STD_LOGIC := '1';
      updown: IN STD_LOGIC := '1';
      sload: IN STD_LOGIC := '0';
      sset: IN STD_LOGIC := '0';
      sclr: IN STD_LOGIC := '0';
      aload: IN STD_LOGIC := '0';
      aset: IN STD_LOGIC := '0';
      aclr: IN STD_LOGIC := '0';
      cout: OUT STD_LOGIC := '0';
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
COMPONENT par_chk
	PORT(
		AD : IN std_logic_vector(31 downto 0);
		nCBE : IN std_logic_vector(3 downto 0);
		adr_par : IN std_logic;
		par : IN std_logic;
		clk : IN std_logic;
		clk_en : IN std_logic;
		perr_chk : IN std_logic;
		serr_ena : IN std_logic;          
		PAR_ERR : OUT std_logic;
		SERR_OUT : OUT std_logic
		);
END COMPONENT;
COMPONENT counters
	PORT(
		clk133 : IN std_logic;
		cycle : IN std_logic_vector(3 downto 0);
		RST : IN std_logic_vector(2 downto 0);
        GC_AUTOMON: IN STD_LOGIC;
        UMON_ENA: IN STD_LOGIC_VECTOR (2 downto 0);
        UMON_SCLR: IN STD_LOGIC_VECTOR (2 downto 0);
		WCOUNT : IN std_logic_vector(2 downto 0);
		trailer : IN std_logic_vector(2 downto 0);
		ADR : IN std_logic_vector(5 downto 0);          
		cntr_data : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
signal GC_DAS : std_logic;
signal GC_IDMATCH_i : std_logic;
signal GC_TSTRIP_i : std_logic;
signal GC_MERGE : std_logic;
signal GC_SINGLE : std_logic;
signal GC_RST : std_logic;
signal GC_AUTOMON_i : std_logic;
signal GC_UMON : std_logic;
signal GC_UMON_DL : std_logic;
signal UMON_ENA : std_logic_vector(2 downto 0);
signal UMON_ENB : std_logic_vector(2 downto 0);
signal UMON_SCLR : std_logic_vector(2 downto 0);
signal M_UERR : std_logic_vector(26 downto 0);
signal M_CERR : std_logic_vector(26 downto 0);
signal M_BADID : std_logic_vector(26 downto 0);
signal M_FULL : std_logic_vector(2 downto 0);
signal M_OVER : std_logic_vector(2 downto 0);
signal R_UERR : std_logic_vector(26 downto 0);
signal R_CERR : std_logic_vector(26 downto 0);
signal R_BADID : std_logic_vector(26 downto 0);
signal R6ch : std_logic_vector(31 downto 0);
signal CC_RST : std_logic_vector(2 downto 0);
signal RST : std_logic_vector(2 downto 0);
signal clk33_dv2 : std_logic;
signal clk33_dv2q : std_logic;
signal cycle : std_logic_vector(3 downto 0);
signal ADI : std_logic_vector(31 downto 0);
signal ADO : std_logic_vector(31 downto 0);
signal ADR : std_logic_vector(5 downto 0);
signal ADDR : std_logic_vector(8 downto 0);
signal nCBEQ : std_logic_vector(3 downto 0);
signal ID_EQ : std_logic;
signal IDS : std_logic;
signal FRAME_0 : std_logic;
signal FR : std_logic_vector(1 downto 0);
signal BAR : std_logic_vector(16 downto 0);
signal FR11 : std_logic;
signal FR13 : std_logic;
signal FR13fifo : std_logic;
signal WR : std_logic;
signal DCC_reset : std_logic;
signal DCC_reset_cntr : std_logic_vector(3 downto 0);
signal MY_CNF : std_logic;
signal MY_FIFO : std_logic;
signal CONF_LOAD : std_logic;
signal WR_CNF : std_logic;
signal WR_4h : std_logic;
signal WR_10h : std_logic;
signal WR_40h : std_logic;
signal LOAD_4h : std_logic;
signal LOAD_10h : std_logic;
signal LOAD_40h : std_logic;
signal R4h : std_logic_vector(31 downto 0);
signal R10h : std_logic_vector(31 downto 0);
signal R40h : std_logic_vector(31 downto 0);
signal R44h : std_logic_vector(31 downto 0);
signal mem_en : std_logic;
signal sel_mem_a : std_logic;
signal sel_mem_b : std_logic;
signal AD_OE : std_logic;
signal PAR_ADO : std_logic_vector(10 downto 0);
signal PAR_GEN : std_logic;
signal PAR_OE : std_logic;
signal PERR_OE : std_logic;
signal PAR_ERR : std_logic;
signal SSE : std_logic;
signal DPE : std_logic;
signal serr_en : std_logic;
signal perr_en : std_logic;
signal perr_chk : std_logic;
signal ins_wait : std_logic;
signal TRDY : std_logic;
signal TRDY_MY : std_logic;
signal TRDY_cycle3 : std_logic;
signal STOP_MY : std_logic;
signal DEVSEL_MY : std_logic;
signal DEVSEL_OE : std_logic;
signal LAST_T : std_logic;
signal FIN_T : std_logic;
signal SERR_OUT : std_logic;
signal mis_match : std_logic_vector(3 downto 0);
signal E_BADID : std_logic;
signal GS_BLOCK : std_logic;
signal GS_EMPTY : std_logic;
signal GS_FULL : std_logic;
signal GS_HEADER : std_logic;
signal GS_OVER : std_logic;
signal GS_CLOW : std_logic_vector(1 downto 0);
signal GS_CHIGH : std_logic_vector(1 downto 0);
signal GS_TRAILER : std_logic;
signal CC_ENA : std_logic_vector(2 downto 0);
signal CS_BLOCK : std_logic_vector(2 downto 0);
signal CS_HEADER : std_logic_vector(2 downto 0);
signal dxfr : std_logic;
signal inc_fifo_ra : std_logic;
signal TRLR : std_logic;
signal TRLRq : std_logic;
signal HDR : std_logic;
signal block_a : std_logic_vector(16 downto 0);
signal block_b : std_logic_vector(16 downto 0);
signal block_c : std_logic_vector(16 downto 0);
signal header_a : std_logic_vector(6 downto 0);
signal header_b : std_logic_vector(6 downto 0);
signal header_c : std_logic_vector(6 downto 0);
signal header_down : std_logic_vector(2 downto 0);
signal ec_block : std_logic_vector(2 downto 0);
signal ec_header : std_logic_vector(2 downto 0);
signal trailer_di : std_logic;
signal wr_fifo_dl : std_logic_vector(2 downto 0);
signal header_out : std_logic_vector(2 downto 0);
signal header_out_dl : std_logic_vector(2 downto 0);
signal merge : std_logic_vector(2 downto 0);
signal next_ch : std_logic_vector(1 downto 0);
signal pci_ch_i : std_logic_vector(1 downto 0);
signal ch0_rp : std_logic_vector(5 downto 0);
signal ch1_rp : std_logic_vector(5 downto 0);
signal ch2_rp : std_logic_vector(5 downto 0);
signal fifo_ra : std_logic_vector(7 downto 0);
signal fifo_do : std_logic_vector(32 downto 0);
signal mux_sel : std_logic_vector(5 downto 0);
signal mux2a : std_logic_vector(31 downto 0);
signal mux2b : std_logic_vector(31 downto 0);
signal mux2c : std_logic_vector(31 downto 0);
signal mux2d : std_logic_vector(31 downto 0);
signal fifo_data : std_logic_vector(31 downto 0);
signal reg_data : std_logic_vector(31 downto 0);
signal cntr_data : std_logic_vector(31 downto 0);
begin
gc_automon <= GC_AUTOMON_i;
rst_out <= RST;
gc_idmatch <= GC_IDMATCH_i;
gc_tstrip <= GC_TSTRIP_i;
pci_ch <= pci_ch_i;
process(clk33)
begin
  if(clk33'event and clk33 = '1')then
    clk33_dv2 <= not clk33_dv2;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    clk33_dv2q <= clk33_dv2;
	cycle(0) <= cycle(3);
	cycle(1) <= clk33_dv2q xor clk33_dv2;
	cycle(3 downto 2) <= cycle(2 downto 1);
  end if;
end process;
process(clk133,nRST)
begin
  if(nRST = '0')then
    RST <= "111";
  elsif(clk133'event and clk133 = '1')then
    if(cycle(3) = '1')then
      RST(0) <= CC_RST(0) or GC_RST or DCC_reset;
      RST(1) <= CC_RST(1) or GC_RST or DCC_reset;
      RST(2) <= CC_RST(2) or GC_RST or DCC_reset;
	end if;
  end if;
end process;
AD <= ADO when AD_OE = '1' else (others => 'Z');
PAR <= PAR_GEN when PAR_OE = '1' else 'Z';
nPERR <= not PAR_ERR when PERR_OE = '1' else 'Z';
nSERR <= '0'  when SERR_OUT = '1' else 'Z';
nDEVSEL <= not DEVSEL_MY when DEVSEL_OE = '1' else 'Z';
nTRDY <= not TRDY_MY  when DEVSEL_OE = '1' else 'Z';
nSTOP <= not STOP_MY  when DEVSEL_OE = '1' else 'Z';
i_par_chk: par_chk PORT MAP(
		AD => ADI,
		nCBE => nCBEQ,
		adr_par => FR(1),
		par => PAR,
		clk => clk133,
		clk_en => cycle(3),
		perr_chk => perr_chk,
		serr_ena => serr_en,
		PAR_ERR => PAR_ERR,
		SERR_OUT => SERR_OUT
	);
fifo_data <= fifo_do(31 downto 20) & (fifo_do(19) or (E_BADID and fifo_do(32))) & fifo_do(18 downto 0);
process(clk133,mem_en)
begin
  if(mem_en = '0')then
    sel_mem_a <= '0';
    sel_mem_b <= '0';
  elsif(clk133'event and clk133 = '1')then
	if(ADI(31 downto 23) = BAR(16 downto 8))then
	  sel_mem_a <= '1';
    else
      sel_mem_a <= '0';
	end if;
	if(ADI(22 downto 15) = BAR(7 downto 0) and nCBEQ(2 downto 1) = "11")then
	  sel_mem_b <= '1';
    else
      sel_mem_b <= '0';
	end if;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    TRDY_cycle3 <= TRDY_MY and not STOP_MY and cycle(2);
    if(FR13 = '1' or (nIRDY = '0' and TRDY_cycle3 = '1'))then
      ADO <= reg_data or cntr_data or fifo_data;
    end if;
	if(cycle(3) = '1')then
	  ADI <= AD;
	  nCBEQ <= nCBE;
	  IDS <= IDSEL;
	  FRAME_0 <= not nFRAME;
	  FR(1) <= FR(0);
	  PERR_OE <= DEVSEL_OE and WR;
	end if;
	FR11 <= FR(1) and cycle(0);
	FR13 <= FR(1) and not WR and cycle(2);
	FR13fifo <= FR(1) and not WR and MY_FIFO and cycle(2);
	if(FR11 = '1')then
	  WR <= nCBEQ(0);
	  ADDR <= ADI(8 downto 0);
    end if;
	if(cycle(3) = '1' and FR(0) = '1')then
	  ADR <= AD(14) & AD(6 downto 2);
	elsif(FR13 = '1' or (nIRDY = '0' and TRDY_cycle3 = '1'))then
	  ADR(4 downto 0) <= ADR(4 downto 0) + 1;
	end if;
    perr_chk <= WR and perr_en;
  end if;
end process;
FR(0) <= not nFRAME and not FRAME_0;
FIN_T <= nFRAME and not nIRDY and (TRDY_MY or STOP_MY);
process(clk133,nRST)
begin
  if(nRST = '0')then
    MY_CNF <= '0';
    DCC_reset <= '0';
    MY_FIFO <= '0';
    LAST_T <= '1';
    ins_wait <= '0';
    TRDY <= '0';
    TRDY_MY <= '0';
    STOP_MY <= '0';
    AD_OE <= '0';
    PAR_OE <= '0';
    DEVSEL_OE <= '0';
    DEVSEL_MY <= '0';
  elsif(clk133'event and clk133 = '1')then
 	if(FR11 = '1')then
	  if((IDS = '1' and nCBEQ(3 downto 1) = "101" and ADI(1 downto 0) = "00" and ADI(10 downto 7) = "0000") or 
                        (sel_mem_a = '1' and sel_mem_b = '1' and nCBEQ(3) = '0' and ADI(14) = '0'))then
		MY_CNF <= '1';
      else
		MY_CNF <= '0';
	  end if;
	  if(IDS = '1' and nCBEQ = "1011" and ADI(1 downto 0) = "00" and ADI(10 downto 7) = "0001") then 
		DCC_reset <= '1';
      elsif(DCC_reset_cntr(3) = '1')then
		DCC_reset <= '0';
	  end if;
	  if(sel_mem_a = '1' and sel_mem_b = '1' and nCBEQ(0) = '0' and ADI(14) = '1')then
		MY_FIFO <= '1';
      else
		MY_FIFO <= '0';
	  end if;
	end if;
	if(cycle(2) = '1')then
      if(LAST_T = '1')then
        TRDY <= '0';
	  else
        TRDY <= TRDY or (FR(1) and (MY_CNF or (MY_FIFO and (not fifo_ra(7) or not fifo_ra(6)))));
	  end if;
    end if;
	if(cycle(3) = '1')then
	  LAST_T <= FIN_T;
      ins_wait <= (lfifo_high(2) and lfifo_high(1) and lfifo_high(0)) and MY_FIFO;
	  if(FIN_T = '1' or (nIRDY = '0' and STOP_MY = '1'))then
	    TRDY_MY <= '0';
	  else
	    TRDY_MY <= TRDY and (not ins_wait or not TRDY_MY or nIRDY or fifo_do(32));
	  end if;
	  if(FIN_T = '1' or GC_SINGLE = '1')then
	    STOP_MY <= '0';
	  elsif(MY_FIFO = '1' and ((fifo_do(32) = '1' and next_ch = "11" and TRDY_MY = '1') or 
                               (TRDY = '0' and FR(1) = '1')))then
	    STOP_MY <= '1';
	  end if;
	  if(FIN_T = '1')then
	    AD_OE <= '0';
	  elsif(WR = '0' and (MY_CNF = '1' or MY_FIFO = '1') and FR(1) = '1')then
	    AD_OE <= '1';
	  end if;
	  PAR_OE <= AD_OE;
	  if(LAST_T = '1' or (nIRDY = '1' and nFRAME = '1'))then
	    DEVSEL_OE <= '0';
	  elsif(FR(1) = '1' and (MY_CNF = '1' or MY_FIFO = '1'))then
	    DEVSEL_OE <= '1';
	  end if;
	  if(FIN_T = '1')then
	    DEVSEL_MY <= '0';
	  elsif(FR(1) = '1' and (MY_CNF = '1' or MY_FIFO = '1'))then
	    DEVSEL_MY <= '1';
	  end if;
	end if;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    for i in 0 to 7 loop
      PAR_ADO(i) <= ADO(i*4) xor ADO(i*4+1) xor ADO(i*4+2) xor ADO(i*4+3);
	end loop;
	PAR_ADO(8) <= PAR_ADO(0) xor PAR_ADO(1) xor PAR_ADO(2) xor PAR_ADO(3);
	PAR_ADO(9) <= PAR_ADO(4) xor PAR_ADO(5) xor PAR_ADO(6) xor PAR_ADO(7);
	PAR_ADO(10) <= PAR_ADO(9) xor PAR_ADO(8);
	if(cycle(3) <= '1')then
	  PAR_GEN <= nCBE(0) xor nCBE(1) xor nCBE(2) xor nCBE(3) xor PAR_ADO(10);
	end if;
  end if;
end process;
process(clk133,nRST)
begin
  if(nRST = '0')then
    inc_fifo_ra <= '0';
    dxfr <= '0';
    pci_read <= "000";
    TRLR <= '0';
    TRLRq <= '0';
  elsif(clk133'event and clk133 = '1')then
	inc_fifo_ra <= FR13fifo or (not nIRDY and not nFRAME and TRDY_cycle3);
	dxfr <= not nIRDY and TRDY_MY and cycle(3);
	pci_read(2) <= inc_fifo_ra and fifo_ra(7) and not fifo_ra(6) and not CS_EMPTY(2);
	pci_read(1) <= inc_fifo_ra and not fifo_ra(7) and fifo_ra(6) and not CS_EMPTY(1);
	pci_read(0) <= inc_fifo_ra and not fifo_ra(7) and not fifo_ra(6) and not CS_EMPTY(0);
    TRLR <= fifo_do(32);
    TRLRq <= TRLR;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    if(DCC_reset = '0')then
      DCC_reset_cntr <= (others => '0');
    elsif(cycle(3) = '1' and DCC_reset_cntr(3) = '0')then
      DCC_reset_cntr <= DCC_reset_cntr + 1;
    end if;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    if(cycle(0) = '1')then
      pci_ch_i <= fifo_ra(7 downto 6);
    end if;
    case fifo_ra(7 downto 6) is
      when "00" => HDR <= header_out(0);
      when "01" => HDR <= header_out(1);
      when "10" => HDR <= header_out(2);
      when others => HDR <= '0';
    end case;
  end if;
end process;
process(clk133)
begin
  if(GC_TSTRIP_i = '1' or GC_IDMATCH_i = '0')then
    E_BADID <= '0';
  elsif(clk133'event and clk133 = '1')then
    for i in mis_match'range loop
      mis_match(i) <= (fifo_do(i*2) xor ADDR(i*2)) or (fifo_do(i*2+1) xor ADDR(i*2+1));
    end loop;
    if(dxfr = '1' and HDR = '1' and mis_match /= "0000")then
      E_BADID <= ADDR(8);
    end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(0) = '1')then
    ch0_rp <= "000000";
  elsif(clk133'event and clk133 = '1')then
	 if(inc_fifo_ra = '1' and fifo_ra(7 downto 6) = "00" and CS_EMPTY(0) = '0')then
	   ch0_rp <= ch0_rp + 1; 
	 end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(1) = '1')then
    ch1_rp <= "000000";
  elsif(clk133'event and clk133 = '1')then
	 if(inc_fifo_ra = '1' and fifo_ra(7 downto 6) = "01" and CS_EMPTY(1) = '0')then
	   ch1_rp <= ch1_rp + 1; 
	 end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(2) = '1')then
    ch2_rp <= "000000";
  elsif(clk133'event and clk133 = '1')then
	 if(inc_fifo_ra = '1' and fifo_ra(7 downto 6) = "10" and CS_EMPTY(2) = '0')then
	   ch2_rp <= ch2_rp + 1; 
	 end if;
  end if;
end process;
process(clk133,CC_ENA)
begin
  if(CC_ENA(0) = '0')then
    merge(0) <= '0';
  elsif(clk133'event and clk133 = '1')then
    if(cycle(3) = '1')then
      if(MY_FIFO = '0')then
        merge(0) <= CS_BLOCK(0);
      else
        merge(0) <= '0';
      end if;
    end if;
  end if;
end process;
process(clk133,CC_ENA)
begin
  if(CC_ENA(1) = '0')then
    merge(1) <= '0';
  elsif(clk133'event and clk133 = '1')then
    if(cycle(3) = '1')then
      if(MY_FIFO = '0')then
        merge(1) <= CS_BLOCK(1);
      else
        merge(1) <= not fifo_ra(6) and merge(1);
      end if;
    end if;
  end if;
end process;
process(clk133,CC_ENA)
begin
  if(CC_ENA(2) = '0')then
    merge(2) <= '0';
  elsif(clk133'event and clk133 = '1')then
    if(cycle(3) = '1')then
      if(MY_FIFO = '0')then
        merge(2) <= CS_BLOCK(2);
      else
        merge(2) <= not fifo_ra(7) and merge(2);
      end if;
    end if;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    if(cycle(3) = '1')then
      next_ch(0) <= (not merge(0) and (merge(1) or not merge(2))) or not GC_MERGE;
      next_ch(1) <= (not merge(0) and not merge(1)) or not GC_MERGE;
      if(GC_DAS = '1')then
        data_av <= CS_HEADER;
      else
        data_av <= CS_BLOCK;
      end if;
    end if;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
	if(FR(1) = '1' and cycle(0) = '1')then
      if(ADI(14) = '0')then
		fifo_ra(7 downto 6) <= "11";
	  elsif(ADI(13 downto 12) /= "11")then
		fifo_ra(7 downto 6) <= ADI(13 downto 12);
      else
		fifo_ra(7 downto 6) <= next_ch;
	  end if;
	elsif(inc_fifo_ra = '1' and TRLR = '1' and GC_SINGLE = '0')then
	  fifo_ra(7 downto 6) <= next_ch;
	end if;
  end if;
end process;
fifo_ra(5 downto 0) <= ch2_rp when fifo_ra(7) = '1' else
                       ch1_rp when fifo_ra(6) = '1' else ch0_rp;
i_pci_fifo: lpm_ram_dp
   port map(
           rdaddress => fifo_ra,
           wraddress => fifo_wa,
           wrclock => clk133,
           rdclock => clk133,
           wren => fifo_we,
           data => fifo_di,
           q => fifo_do);
-- conf registers
process(clk133,nRST)
begin
  if(nRST = '0')then
    WR_CNF <= '0';
    WR_4h <= '0';
    WR_10h <= '0';
    WR_40h <= '0';
    LOAD_4h <= '0';
    LOAD_10h <= '0';
    LOAD_40h <= '0';
    R4h(1) <= '0';
    R4h(6) <= '0';
    R4h(8) <= '0';
    SSE <= '0';
    DPE <= '0';
    R10h(31 downto 15) <= "00000000000000000";
    R40h(3 downto 0) <= "0000";
    R40h(10 downto 5) <= "010000";
    R40h(21 downto 12) <= "1111110000";
    R40h(29 downto 24) <= "111000";
  elsif(clk133'event and clk133 = '1')then
    WR_CNF <= not ADR(3) and not ADR(1) and MY_CNF and WR;
    WR_4h <= WR_CNF and not ADR(4) and not ADR(2) and ADR(0);
    WR_10h <= WR_CNF and not ADR(4) and ADR(2) and not ADR(0);
    WR_40h <= WR_CNF and ADR(4) and not ADR(2) and not ADR(0);
    LOAD_4h <= WR_4h and dxfr;
    LOAD_10h <= WR_10h and dxfr;
    LOAD_40h <= WR_40h and dxfr;
    if(LOAD_4h = '1' and nCBEQ(0) = '0')then
      R4h(1) <= ADI(1);
      R4h(6) <= ADI(6);
    end if;
    if(LOAD_4h = '1' and nCBEQ(1) = '0')then
      R4h(8) <= ADI(8);
    end if;
    if(SERR_OUT = '1')then
      SSE <= '1';
    elsif(LOAD_4h = '1' and nCBEQ(3) = '0' and ADI(30) = '1')then
      SSE <= '0';
    end if;
    if(PAR_ERR = '1')then
      DPE <= '1';
    elsif(LOAD_4h = '1' and nCBEQ(3) = '0' and ADI(31) = '1')then
      DPE <= '0';
    end if;
    if(LOAD_10h = '1' and nCBEQ(1) = '0')then
      R10h(15) <= ADI(15);
    end if;
    if(LOAD_10h = '1' and nCBEQ(2) = '0')then
      R10h(23 downto 16) <= ADI(23 downto 16);
    end if;
    if(LOAD_10h = '1' and nCBEQ(3) = '0')then
      R10h(31 downto 24) <= ADI(31 downto 24);
    end if;
    if(LOAD_40h = '1' and nCBEQ(0) = '0')then
      R40h(3 downto 0) <= ADI(3 downto 0);
      R40h(7 downto 5) <= ADI(7 downto 5);
    end if;
    if(LOAD_40h = '1' and nCBEQ(1) = '0' and ADI(10) = '1')then
      R40h(10) <= '1';
    else
      R40h(10) <= '0';
    end if;
    if(LOAD_40h = '1' and nCBEQ(1) = '0')then
      R40h(9 downto 8) <= ADI(9 downto 8);
      R40h(15 downto 12) <= ADI(15 downto 12);
    end if;
    if(LOAD_40h = '1' and nCBEQ(2) = '0')then
      R40h(21 downto 16) <= ADI(21 downto 16);
    end if;
    if(LOAD_40h = '1' and nCBEQ(3) = '0')then
      R40h(29 downto 24) <= ADI(29 downto 24);
    end if;
    CS_BLOCK <= block_c(16) & block_b(16) & block_a(16);
    CS_HEADER <= header_c(6) & header_b(6) & header_a(6);
  end if;
end process;
R4h(0) <= '0';
mem_en <= R4h(1);
R4h(5 downto 2) <= "0000";
perr_en <= R4h(6);
R4h(7) <= '0';
serr_en <= R4h(8);
R4h(29 downto 9) <= "000100000000000000000";
R4h(30) <= SSE;
R4h(31) <= DPE;
R10h(14 downto 0) <= "000000000000000";
BAR <= R10h(31 downto 15);
GC_OVRL <= R40h(1 downto 0);
GC_TSTRIP_i <= R40h(2);
GC_DAS <= R40h(3);
R40h(4) <= '0';
GC_BLOCK <= R40h(5);
GC_IDMATCH_i <= R40h(6);
GC_MERGE <= R40h(7);
GC_RST <= R40h(8);
GC_UMON <= R40h(10);
R40h(11) <= '0';
GC_SINGLE <= R40h(12);
GC_IFIFO <= R40h(13);
GC_EFIFO <= R40h(14);
GC_RFIFO <= R40h(15);
CC_RXENA <= R40h(18 downto 16);
CC_ECC <= R40h(21 downto 19);
R40h(23 downto 22) <= "00";
CC_RST <= R40h(26 downto 24);
CC_ENA <= R40h(29 downto 27);
R40h(31 downto 30) <= "00";
i_block_a: lpm_counter
   port map(
           clock => clk133,
           clk_en => ec_block(0),
		   updown => header_down(0),
           aclr => rst(0),
           q => block_a);
i_block_b: lpm_counter
   port map(
           clock => clk133,
           clk_en => ec_block(1),
		   updown => header_down(1),
           aclr => rst(1),
           q => block_b);
i_block_c: lpm_counter
   port map(
           clock => clk133,
           clk_en => ec_block(2),
		   updown => header_down(2),
           aclr => rst(2),
           q => block_c);
ec_block <= header_down xor trailer;
i_header_a: lpm_counter
   generic map(LPM_WIDTH => 7)
   port map(
           clock => clk133,
           clk_en => ec_header(0),
		   updown => header_down(0),
           aclr => rst(0),
           q => header_a);
i_header_b: lpm_counter
   generic map(LPM_WIDTH => 7)
   port map(
           clock => clk133,
           clk_en => ec_header(1),
		   updown => header_down(1),
           aclr => rst(1),
           q => header_b);
i_header_c: lpm_counter
   generic map(LPM_WIDTH => 7)
   port map(
           clock => clk133,
           clk_en => ec_header(2),
		   updown => header_down(2),
           aclr => rst(2),
           q => header_c);
ec_header <= header xor header_down;
process(clk133,RST)
begin
  if(RST(0) = '1')then
    header_out(0) <= '1';
  elsif(clk133'event and clk133 = '1')then
    if(dxfr = '1' and pci_ch_i = "00")then
      header_out(0) <= TRLRq;
    end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(1) = '1')then
    header_out(1) <= '1';
  elsif(clk133'event and clk133 = '1')then
    if(dxfr = '1' and pci_ch_i = "01")then
      header_out(1) <= TRLRq;
    end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(2) = '1')then
    header_out(2) <= '1';
  elsif(clk133'event and clk133 = '1')then
    if(dxfr = '1' and pci_ch_i = "10")then
      header_out(2) <= TRLRq;
    end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(0) = '1')then
    header_out_dl(0) <= '1';
    header_down(0) <= '0';
  elsif(clk133'event and clk133 = '1')then
    header_out_dl(0) <= header_out(0);
    header_down(0) <= header_out_dl(0) and not header_out(0);
  end if;
end process;
process(clk133,RST)
begin
  if(RST(1) = '1')then
    header_out_dl(1) <= '1';
    header_down(1) <= '0';
  elsif(clk133'event and clk133 = '1')then
    header_out_dl(1) <= header_out(1);
    header_down(1) <= header_out_dl(1) and not header_out(1);
  end if;
end process;
process(clk133,RST)
begin
  if(RST(2) = '1')then
    header_out_dl(2) <= '1';
    header_down(2) <= '0';
  elsif(clk133'event and clk133 = '1')then
    header_out_dl(2) <= header_out(2);
    header_down(2) <= header_out_dl(2) and not header_out(2);
  end if;
end process;
process(clk133,GC_RST)
begin
  if(GC_RST = '1')then
    GS_TRAILER <= '0';
    GS_CHIGH <= "00";
    GS_CLOW <= "00";
  elsif(clk133'event and clk133 = '1')then
    if(dxfr = '1' and MY_FIFO = '1')then
      GS_CHIGH <= not TRLRq & TRLRq;
      GS_CLOW <= '1' & HDR;
      GS_TRAILER <= TRLRq;
    end if;
  end if;
end process;
R44h(2 downto 0) <= CS_EMPTY;
R44h(5 downto 3) <= CS_FULL;
R44h(7 downto 6) <= "00";
R44h(10 downto 8) <= CS_BLOCK;
R44h(13 downto 11) <= CS_HEADER;
R44h(16 downto 14) <= CS_OVER;
R44h(19 downto 17) <= "000";
R44h(20) <= GS_BLOCK;
R44h(21) <= GS_EMPTY;
R44h(22) <= GS_FULL;
R44h(23) <= GS_HEADER;
R44h(24) <= GS_OVER;
R44h(26 downto 25) <= "00";
R44h(28 downto 27) <= GS_CLOW;
R44h(30 downto 29) <= GS_CHIGH;
R44h(31) <= GS_TRAILER;
GS_EMPTY <= '1' when CS_EMPTY = "111" else '0';
GS_FULL <= '0' when CS_FULL = "000" else '1';
GS_OVER <= '0' when CS_OVER = "000" else '1';
GS_HEADER <= '1' when CS_HEADER = "111" else '0';
GS_BLOCK <= '1' when CS_BLOCK = CC_ENA and CC_ENA /= "000" else '0';
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    GC_AUTOMON_i <= R40h(9);
    GC_UMON_DL <= GC_UMON;
    UMON_ENA(0) <= (GC_UMON and not GC_UMON_DL) or update(0) or R40h(9);
    UMON_ENA(1) <= (GC_UMON and not GC_UMON_DL) or update(1) or R40h(9);
    UMON_ENA(2) <= (GC_UMON and not GC_UMON_DL) or update(2) or R40h(9);
    UMON_ENB(0) <= (GC_UMON and not GC_UMON_DL) or update(0);
    UMON_ENB(1) <= (GC_UMON and not GC_UMON_DL) or update(1);
    UMON_ENB(2) <= (GC_UMON and not GC_UMON_DL) or update(2);
    UMON_SCLR(0) <= (GC_UMON and not GC_UMON_DL) or update(0) or RST(0);
    UMON_SCLR(1) <= (GC_UMON and not GC_UMON_DL) or update(1) or RST(1);
    UMON_SCLR(2) <= (GC_UMON and not GC_UMON_DL) or update(2) or RST(2);
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    if(UMON_SCLR(0) = '1' )then
      M_CERR(8 downto 0) <= "00000000" & cerr(0);
      M_UERR(8 downto 0) <= "00000000" & uerr(0);
      M_BADID(8 downto 0) <= "00000000" & BADID(0);
    else
      if(cerr(0) = '1' and M_CERR(8) = '0')then
        M_CERR(8 downto 0) <= M_CERR(8 downto 0) + 1;
      end if;
      if(uerr(0) = '1' and M_UERR(8) = '0')then
        M_UERR(8 downto 0) <= M_UERR(8 downto 0) + 1;
      end if;
      if(BADID(0) = '1' and M_BADID(8) = '0')then
        M_BADID(8 downto 0) <= M_BADID(8 downto 0) + 1;
      end if;
    end if;
    if(UMON_SCLR(1) = '1' )then
      M_CERR(17 downto 9) <= "00000000" & cerr(1);
      M_UERR(17 downto 9) <= "00000000" & uerr(1);
      M_BADID(17 downto 9) <= "00000000" & BADID(1);
    else
      if(cerr(1) = '1' and M_CERR(17) = '0')then
        M_CERR(17 downto 9) <= M_CERR(17 downto 9) + 1;
      end if;
      if(uerr(1) = '1' and M_UERR(17) = '0')then
        M_UERR(17 downto 9) <= M_UERR(17 downto 9) + 1;
      end if;
      if(BADID(1) = '1' and M_BADID(17) = '0')then
        M_BADID(17 downto 9) <= M_BADID(17 downto 9) + 1;
      end if;
    end if;
    if(UMON_SCLR(2) = '1' )then
      M_CERR(26 downto 18) <= "00000000" & cerr(2);
      M_UERR(26 downto 18) <= "00000000" & uerr(2);
      M_BADID(26 downto 18) <= "00000000" & BADID(2);
    else
      if(cerr(2) = '1' and M_CERR(26) = '0')then
        M_CERR(26 downto 18) <= M_CERR(26 downto 18) + 1;
      end if;
      if(uerr(2) = '1' and M_UERR(26) = '0')then
        M_UERR(26 downto 18) <= M_UERR(26 downto 18) + 1;
      end if;
      if(BADID(2) = '1' and M_BADID(26) = '0')then
        M_BADID(26 downto 18) <= M_BADID(26 downto 18) + 1;
      end if;
    end if;
--    R6ch(8 downto 6) <= lfifo_high or (R6ch(8 downto 6) and not UMON_SCLR);
    M_FULL <= CS_FULL or (M_FULL and not UMON_SCLR);
    M_OVER <= CS_OVER or (M_OVER and not UMON_SCLR);
  end if;
end process;
process(clk133,RST)
begin
  if(RST(0) = '1')then
    R_CERR(8 downto 0) <= "000000000";
    R_UERR(8 downto 0) <= "000000000";
    R_BADID(8 downto 0) <= "000000000";
    R6ch(0) <= '0';
    R6ch(3) <= '0';
  elsif(clk133'event and clk133 = '1')then
    if(UMON_ENA(0) = '1')then
      R_CERR(8 downto 0) <= M_CERR(8 downto 0);
      R_UERR(8 downto 0) <= M_UERR(8 downto 0);
      R_BADID(8 downto 0) <= M_BADID(8 downto 0);
      R6ch(0) <= M_FULL(0);
      R6ch(3) <= M_OVER(0);
    end if;
  end if;
end process;
--R6ch(31 downto 9) <= "00000000000000000000000";
R6ch(31 downto 6) <= "00000000000000000000000000";
process(clk133,RST)
begin
  if(RST(1) = '1')then
    R_CERR(17 downto 9) <= "000000000";
    R_UERR(17 downto 9) <= "000000000";
    R_BADID(17 downto 9) <= "000000000";
    R6ch(1) <= '0';
    R6ch(4) <= '0';
  elsif(clk133'event and clk133 = '1')then
    if(UMON_ENA(1) = '1')then
      R_CERR(17 downto 9) <= M_CERR(17 downto 9);
      R_UERR(17 downto 9) <= M_UERR(17 downto 9);
      R_BADID(17 downto 9) <= M_BADID(17 downto 9);
      R6ch(1) <= M_FULL(1);
      R6ch(4) <= M_OVER(1);
    end if;
  end if;
end process;
process(clk133,RST)
begin
  if(RST(2) = '1')then
    R_CERR(26 downto 18) <= "000000000";
    R_UERR(26 downto 18) <= "000000000";
    R_BADID(26 downto 18) <= "000000000";
    R6ch(2) <= '0';
    R6ch(5) <= '0';
  elsif(clk133'event and clk133 = '1')then
    if(UMON_ENA(2) = '1')then
      R_CERR(26 downto 18) <= M_CERR(26 downto 18);
      R_UERR(26 downto 18) <= M_UERR(26 downto 18);
      R_BADID(26 downto 18) <= M_BADID(26 downto 18);
      R6ch(2) <= M_FULL(2);
      R6ch(5) <= M_OVER(2);
    end if;
  end if;
end process;
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    mux_sel(0) <= ADR(0);
    mux_sel(1) <= ADR(4) and not ADR(3) and not ADR(2) and not ADR(1);
    mux_sel(2) <= ADR(4) and not ADR(3) and not ADR(2) and ADR(1);
    if(mux_sel(1) = '0')then
      mux2a <= (others => '0');
    elsif(mux_sel(0) = '0')then
      mux2a <= R40h;
    else
      mux2a <= R44h;
    end if;
    if(mux_sel(2) = '0')then
      mux2b <= (others => '0');
    elsif(mux_sel(0) = '0')then
      mux2b <= "00000" & R_CERR(26 downto 0);
    else
      mux2b <= "00000" & R_UERR(26 downto 0);
    end if;
    mux_sel(3) <= not ADR(3) and ADR(2) and not ADR(1) and not ADR(0);
    mux_sel(4) <= ADR(4);
    if(mux_sel(3) = '0')then
      mux2c <= (others => '0');
    elsif(mux_sel(4) = '0')then
      mux2c <= R10h;
    else
      mux2c <= "00000" & R_BADID;
    end if;
    if(ADR(4 downto 0) = "00001" or ADR(4 downto 0) = "11011")then
      mux_sel(5) <= '1';
    else
      mux_sel(5) <= '0';
    end if;
    if(mux_sel(5) = '0')then
      mux2d <= (others => '0');
    elsif(mux_sel(4) = '0')then
      mux2d <= R4h;
    else
      mux2d <= R6ch;
    end if;
  end if;
end process;
process(clk133,ADR)
begin
  if(ADR(5) = '1')then
    reg_data <= (others => '0');
  elsif(clk133'event and clk133 = '1')then
    reg_data <= mux2a or mux2b or mux2c or mux2d;
  end if;
end process;
i_counters: counters PORT MAP(
		clk133 => clk133,
		cycle => cycle,
		RST => RST,
		GC_AUTOMON => GC_AUTOMON_i,
		UMON_ENA => UMON_ENB,
		UMON_SCLR => UMON_SCLR,
		WCOUNT => WCOUNT,
		trailer => trailer,
		ADR => ADR,
		cntr_data => cntr_data
	);
end my_arch;
