library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity ram is
    Port ( clk : in std_logic;
           CC_RST : in std_logic_vector(2 downto 0);
-- lvds input signals
           ch_dav : in std_logic_vector(2 downto 0);
           ch0_data : in std_logic_vector(32 downto 0);
           ch1_data : in std_logic_vector(32 downto 0);
           ch2_data : in std_logic_vector(32 downto 0);
           ch_read : out std_logic_vector(2 downto 0);
-- sram signals
-- high read, low write
           nWR : out std_logic;
-- chip enable
           RCS : out std_logic;
           adr : out std_logic_vector(18 downto 0);
           sd : in std_logic_vector(35 downto 0);
           sq : out std_logic_vector(35 downto 0);
-- strl and status
           GC_IFIFO : in std_logic;
           GC_RFIFO : in std_logic;
           GC_OVRL : in std_logic_vector(1 downto 0);
           CS_OVER: out std_logic_vector(2 downto 0);
           CS_FULL: out std_logic_vector(2 downto 0);
-- pci fifo signals
           pfifo_read : in std_logic_vector(2 downto 0);
           pfifo_ch : in std_logic_vector(1 downto 0);
           CS_EMPTY: out std_logic_vector(2 downto 0);
           pfifo_header : out std_logic_vector(2 downto 0);
           pfifo_we : out std_logic;
           pfifo_wa : out std_logic_vector(7 downto 0);
           pfifo_data : out std_logic_vector(32 downto 0)
		);
end ram;

architecture my_arch of ram is
COMPONENT lpm_counter
   GENERIC (LPM_WIDTH: POSITIVE := 16;
      LPM_MODULUS: NATURAL := 0;
      LPM_DIRECTION: STRING := "UNUSED";
      LPM_AVALUE: STRING := "UNUSED";
      LPM_SVALUE: STRING := "UNUSED";
      LPM_PVALUE: STRING := "UNUSED";
      LPM_TYPE: STRING := "LPM_COUNTER";
      LPM_HINT: STRING := "UNUSED");
   PORT (data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0) := (OTHERS => '0');
      clock: IN STD_LOGIC;
      cin: IN STD_LOGIC := '0';
      clk_en: IN STD_LOGIC := '1';
      cnt_en: IN STD_LOGIC := '1';
      updown: IN STD_LOGIC := '1';
      sload: IN STD_LOGIC := '0';
      sset: IN STD_LOGIC := '0';
      sclr: IN STD_LOGIC := '0';
      aload: IN STD_LOGIC := '0';
      aset: IN STD_LOGIC := '0';
      aclr: IN STD_LOGIC := '0';
      cout: OUT STD_LOGIC := '0';
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
signal high_lfifo: std_logic;
signal rd_sram_dl: std_logic_vector(3 downto 0);
signal wr_sram_dl: std_logic_vector(3 downto 0);
signal sq_t: std_logic;
signal ch_read_i: std_logic_vector(2 downto 0);
signal en_write: std_logic;
signal rst: std_logic;
signal RCSq: std_logic;
signal nWRq: std_logic;
signal pfifo_rqst: std_logic;
signal rd_sram: std_logic;
signal wr_thru: std_logic;
signal sel_data: std_logic_vector(1 downto 0);
signal sqr: std_logic_vector(32 downto 0);
signal sddr: std_logic_vector(32 downto 0);
signal w_data: std_logic_vector(32 downto 0);
signal ch2_data_dl: std_logic_vector(32 downto 0);
signal mux_data: std_logic_vector(32 downto 0);
signal sel_adr: std_logic_vector(1 downto 0);
signal sel_adr_dl: std_logic_vector(1 downto 0);
signal ch0_wp: std_logic_vector(15 downto 0);
signal ch1_wp: std_logic_vector(15 downto 0);
signal ch2_wp: std_logic_vector(15 downto 0);
signal ch0_wc: std_logic_vector(15 downto 0);
signal ch1_wc: std_logic_vector(15 downto 0);
signal ch2_wc: std_logic_vector(15 downto 0);
signal ec_ch_wc: std_logic_vector(2 downto 0);
signal ch_wc_low: std_logic_vector(2 downto 0);
signal ch_empty: std_logic_vector(2 downto 0);
signal wr2pfifo: std_logic_vector(2 downto 0);
signal en_wr2sram: std_logic_vector(2 downto 0);
signal sel_rp : std_logic_vector(2 downto 0);
signal ch0_rp: std_logic_vector(15 downto 0);
signal ch1_rp: std_logic_vector(15 downto 0);
signal ch2_rp: std_logic_vector(15 downto 0);
signal mux_adr_a: std_logic_vector(17 downto 0);
signal mux_adr_b: std_logic_vector(17 downto 0);
signal rp: std_logic_vector(17 downto 0);
signal adrq: std_logic_vector(17 downto 0);
signal pfifo_wa_q: std_logic_vector(7 downto 0);
signal pfifo_wa_q2: std_logic_vector(7 downto 0);
signal pfifo_wa_q3: std_logic_vector(7 downto 0);
signal pfifo_wa_i: std_logic_vector(7 downto 0);
signal trailer: std_logic;
signal pfifo_write: std_logic_vector(2 downto 0);
signal pfifo_next_header: std_logic_vector(2 downto 0);
signal pfifo_full: std_logic_vector(2 downto 0);
signal pfifo_rqst_a: std_logic_vector(2 downto 0);
signal pfifo_low: std_logic_vector(2 downto 0);
signal pfifo_we_i: std_logic_vector(3 downto 0);
signal ec_pfifo_wc: std_logic_vector(2 downto 0);
signal ch0_pfifo_wc: std_logic_vector(5 downto 0);
signal ch1_pfifo_wc: std_logic_vector(5 downto 0);
signal ch2_pfifo_wc: std_logic_vector(5 downto 0);
signal aaa: std_logic_vector(5 downto 0);
begin
pfifo_we <= pfifo_we_i(3);
pfifo_wa <= pfifo_wa_i;
pfifo_data <= sddr;
-- rqst queue
process(clk)
begin
  if(clk'event and clk = '1')then
-- read sram has higher priority and one turn-around cycle is always added between read and write
    if((sel_rp or pfifo_rqst_a or (pfifo_low and not ch_empty)) = "000")then  
      en_write <= '1';
    else
      en_write <= '0';
    end if;
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(0) = '1')then
    ch_read_i(0) <= '0';
  elsif(clk'event and clk = '1')then
-- write priority between LVDS channels is rotating. When ch_read_i = "000" and ch_dav = "111", read channel 0
    ch_read_i(0) <= not ch_read_i(0) and ch_dav(0) and en_write and (ch_read_i(2) or not ch_dav(2) or (not ch_read_i(1) and ch_dav(1)));
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(1) = '1')then
    ch_read_i(1) <= '0';
  elsif(clk'event and clk = '1')then
-- write priority between LVDS channels is rotating
    ch_read_i(1) <= not ch_read_i(1) and ch_dav(1) and en_write and (ch_read_i(0) or not ch_dav(0));
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(2) = '1')then
    ch_read_i(2) <= '0';
  elsif(clk'event and clk = '1')then
-- write priority between LVDS channels is rotating
    ch_read_i(2) <= not ch_read_i(2) and ch_dav(2) and en_write and (ch_read_i(1) or not ch_dav(1));
  end if;
end process;
ch_read <= ch_read_i;
sel_adr(0) <= rd_sram or ch_read_i(1);
process(clk)
begin
  if(clk'event and clk = '1')then
-- signals in this section are difficult to handle for individual channel resets,
-- but if channel reset stays long enough, it works without resetting
    rd_sram <= pfifo_rqst;
    sel_adr(1) <= rd_sram or ch_read_i(2);
    nWR <= rd_sram_dl(1);
    RCS <= RCSq;
-- enable sram for read or write
    RCSq <= rd_sram_dl(0) or wr_sram_dl(0);
    wr_sram_dl(0) <= (ch_read_i(0) and en_wr2sram(0)) or (ch_read_i(1) and en_wr2sram(1)) or (ch_read_i(2) and en_wr2sram(2));
    wr_sram_dl(3 downto 1) <= wr_sram_dl(2 downto 0);
-- one cycle each before and after read cycles SQ outputs are tri-stated to prevent bus contention
    sq_t <= rd_sram_dl(1) or rd_sram_dl(2) or rd_sram_dl(3);
    for i in aaa'range loop
      aaa(i) <= sq_t;
    end loop;
    if(sel_adr(0) = '0')then
      mux_adr_a <= "00" & ch0_wp;
      mux_adr_b <= "10" & ch2_wp;
    else
      mux_adr_a <= "01" & ch1_wp;
      mux_adr_b <= rp;
    end if;
    if(sel_adr(1) = '0')then
      adrq <= mux_adr_a;
    else
      adrq <= mux_adr_b;
    end if;
    adr <= '0' & adrq;
    sqr <= w_data;
    wr2pfifo <= ch_read_i and not en_wr2sram;
    wr_thru <= wr2pfifo(0) or wr2pfifo(1) or wr2pfifo(2);
    pfifo_we_i(3 downto 0) <= pfifo_we_i(2 downto 0) & (wr_thru or rd_sram_dl(1));
    rd_sram_dl <= rd_sram_dl(2 downto 0) & rd_sram; 
    pfifo_rqst <= pfifo_rqst_a(2) or pfifo_rqst_a(1) or pfifo_rqst_a(0);
	sel_rp(2) <= pfifo_rqst_a(2) and (pfifo_ch(1) or (not pfifo_rqst_a(1) and not pfifo_rqst_a(0)));
    if((pfifo_ch(1) = '0' or pfifo_rqst_a(2) = '0') and (pfifo_ch(0) = '1' or pfifo_rqst_a(0) = '0'))then
	  sel_rp(1) <= pfifo_rqst_a(1);
	else
	  sel_rp(1) <= '0';
	end if;
    if((pfifo_ch(1) = '0' or pfifo_rqst_a(2) = '0') and (pfifo_ch(0) = '0' or pfifo_rqst_a(1) = '0'))then
	  sel_rp(0) <= pfifo_rqst_a(0);
	else
	  sel_rp(0) <= '0';
	end if;
    pfifo_wa_q <= adrq(17 downto 16) & adrq(5 downto 0);
    pfifo_wa_q2 <= pfifo_wa_q;
    pfifo_wa_q3 <= pfifo_wa_q2;
    pfifo_wa_i <= pfifo_wa_q3;
    sel_adr_dl <= sel_adr;
    sel_data <= sel_adr_dl;
    ch2_data_dl <= ch2_data;
    if(sel_data(1) = '1')then
      w_data <= ch2_data_dl;
    else
      w_data <= mux_data;
    end if;
    if(sel_data(0) = '1')then
      mux_data <= ch1_data;
    else
      mux_data <= ch0_data;
    end if;
    sddr <= sd(32 downto 0);
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    trailer <= sddr(32);
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    case sel_rp(2 downto 1) is
      when "00" => rp <= "00" & ch0_rp; 
      when "01" => rp <= "01" & ch1_rp; 
      when others => rp <= "10" & ch2_rp;
    end case; 
  end if;
end process;
i_ch0_pfifo_wc: lpm_counter
   generic map(LPM_WIDTH => 6)
   port map(
           clock => clk,
           clk_en => ec_pfifo_wc(0),
		   updown => pfifo_write(0),
           aclr => cc_rst(0),
           q => ch0_pfifo_wc);
ec_pfifo_wc(0) <= pfifo_read(0) xor pfifo_write(0);
i_ch1_pfifo_wc: lpm_counter
   generic map(LPM_WIDTH => 6)
   port map(
           clock => clk,
           clk_en => ec_pfifo_wc(1),
		   updown => pfifo_write(1),
           aclr => cc_rst(1),
           q => ch1_pfifo_wc);
ec_pfifo_wc(1) <= pfifo_read(1) xor pfifo_write(1);
i_ch2_pfifo_wc: lpm_counter
   generic map(LPM_WIDTH => 6)
   port map(
           clock => clk,
           clk_en => ec_pfifo_wc(2),
		   updown => pfifo_write(2),
           aclr => cc_rst(2),
           q => ch2_pfifo_wc);
ec_pfifo_wc(2) <= pfifo_read(2) xor pfifo_write(2);
process(clk,cc_rst)
begin
  if(cc_rst(0) = '1')then
    ch_wc_low(0) <= '1';
    ch_empty(0) <= '1';
    CS_EMPTY(0) <= '1';
    pfifo_write(0) <= '0';
    pfifo_header(0) <= '0';
    pfifo_next_header(0) <= '1';
    pfifo_full(0) <= '0';
    pfifo_low(0) <= '1';
    pfifo_rqst_a(0) <= '0';
    en_wr2sram(0) <= '0';
    ch0_wp <= (others => '0');
    ch0_rp <= (others => '0');
  elsif(clk'event and clk = '1')then
    if(ch0_wc(15 downto 2) = "00000000000000")then
      ch_wc_low(0) <= '1';
    else
      ch_wc_low(0) <= '0';
    end if;
    if((ch_read_i(0) = '1' and en_wr2sram(0) = '1') or GC_RFIFO = '1')then
      ch_empty(0) <= '0';
    elsif(ch_wc_low(0) = '1' and ch0_wc(2 downto 1) = "00" and sel_rp(0) = '1')then
      ch_empty(0) <= '1';
    end if;
    if(ch0_pfifo_wc = "000000")then
      CS_EMPTY(0) <= '1';
    else
      CS_EMPTY(0) <= '0';
    end if;
    pfifo_write(0) <= not pfifo_wa_q3(7) and not pfifo_wa_q3(6) and pfifo_we_i(2);
    if(pfifo_write(0) = '1')then
      pfifo_next_header(0) <= trailer;
    end if;
    pfifo_header(0) <= pfifo_next_header(0) and pfifo_write(0);
-- pfifo almost full
    pfifo_full(0) <= (ch0_pfifo_wc(5) and ch0_pfifo_wc(4)) and not GC_IFIFO;
-- pfifo almost empty and sram not empty
    if(pfifo_full(0) = '1')then
      pfifo_low(0) <= '0';
    elsif(ch0_pfifo_wc(5 downto 4) = "00")then
      pfifo_low(0) <= '1';
    end if;
-- set request when pfifo enters into low, reset request when sram empty or pfifo almost full
    if(pfifo_low(0) = '1' and (ch_wc_low(0) = '0' or ch0_wc(2) = '1' or (ch0_wc(1) = '1' and (ch0_wc(0) = '1' or sel_rp(0) = '0')) or
              ((pfifo_rqst_a(0) = '0' or ch_read_i(0) = '1') and ch0_wc(0) = '1' and sel_rp(0) = '0')))then
      pfifo_rqst_a(0) <= '1';
    else
      pfifo_rqst_a(0) <= '0';
    end if;
--
    en_wr2sram(0) <= pfifo_full(0) or not ch_empty(0);
    if(ch_read_i(0) = '1')then
      ch0_wp <= ch0_wp + 1;
    end if;
    if(sel_rp(0) = '1' or wr2pfifo(0) = '1')then
      ch0_rp <= ch0_rp + 1;
    end if;
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(1) = '1')then
    ch_wc_low(1) <= '1';
    ch_empty(1) <= '1';
    CS_EMPTY(1) <= '1';
    pfifo_write(1) <= '0';
    pfifo_header(1) <= '0';
    pfifo_next_header(1) <= '1';
    pfifo_full(1) <= '0';
    pfifo_low(1) <= '0';
    pfifo_rqst_a(1) <= '0';
    en_wr2sram(1) <= '0';
    ch1_wp <= (others => '0');
    ch1_rp <= (others => '0');
  elsif(clk'event and clk = '1')then
    if(ch1_wc(15 downto 2) = "00000000000000")then
      ch_wc_low(1) <= '1';
    else
      ch_wc_low(1) <= '0';
    end if;
    if((ch_read_i(1) = '1' and en_wr2sram(1) = '1') or GC_RFIFO = '1')then
      ch_empty(1) <= '0';
    elsif(ch_wc_low(1) = '1' and ch1_wc(2 downto 1) = "00" and sel_rp(1) = '1')then
      ch_empty(1) <= '1';
    end if;
    if(ch1_pfifo_wc = "000000")then
      CS_EMPTY(1) <= '1';
    else
      CS_EMPTY(1) <= '0';
    end if;
    pfifo_write(1) <= pfifo_wa_q3(6) and pfifo_we_i(2);
    if(pfifo_write(1) = '1')then
      pfifo_next_header(1) <= trailer;
    end if;
    pfifo_header(1) <= pfifo_next_header(1) and pfifo_write(1);
-- pfifo almost full
    pfifo_full(1) <= (ch1_pfifo_wc(5) and ch1_pfifo_wc(4)) and not GC_IFIFO;
-- pfifo almost empty and sram not empty which is checked by en_wr2sram
    if(pfifo_full(1) = '1')then
      pfifo_low(1) <= '0';
    elsif(ch1_pfifo_wc(5 downto 4) = "00")then
      pfifo_low(1) <= '1';
    end if;
-- set request when pfifo enters into low, reset request when sram empty or pfifo almost full
    if(pfifo_low(1) = '1' and (ch_wc_low(1) = '0' or ch1_wc(2) = '1' or (ch1_wc(1) = '1' and (ch1_wc(0) = '1' or sel_rp(1) = '0')) or
              ((pfifo_rqst_a(1) = '0' or ch_read_i(1) = '1') and ch1_wc(0) = '1' and sel_rp(1) = '0')))then
      pfifo_rqst_a(1) <= '1';
    else
      pfifo_rqst_a(1) <= '0';
    end if;
--
    en_wr2sram(1) <= pfifo_full(1) or not ch_empty(1);
    if(ch_read_i(1) = '1')then
      ch1_wp <= ch1_wp + 1;
    end if;
    if(sel_rp(1) = '1' or wr2pfifo(1) = '1')then
      ch1_rp <= ch1_rp + 1;
    end if;
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(2) = '1')then
    ch_wc_low(2) <= '1';
    ch_empty(2) <= '1';
    CS_EMPTY(2) <= '1';
    pfifo_write(2) <= '0';
    pfifo_header(2) <= '0';
    pfifo_next_header(2) <= '1';
    pfifo_full(2) <= '0';
    pfifo_low(2) <= '0';
    pfifo_rqst_a(2) <= '0';
    en_wr2sram(2) <= '0';
    ch2_wp <= (others => '0');
    ch2_rp <= (others => '0');
  elsif(clk'event and clk = '1')then
    if(ch2_wc(15 downto 2) = "00000000000000")then
      ch_wc_low(2) <= '1';
    else
      ch_wc_low(2) <= '0';
    end if;
    if((ch_read_i(2) = '1' and en_wr2sram(2) = '1') or GC_RFIFO = '1')then
      ch_empty(2) <= '0';
    elsif(ch_wc_low(2) = '1' and ch2_wc(2 downto 1) = "00" and sel_rp(2) = '1')then
      ch_empty(2) <= '1';
    end if;
    if(ch2_pfifo_wc = "000000")then
      CS_EMPTY(2) <= '1';
    else
      CS_EMPTY(2) <= '0';
    end if;
    pfifo_write(2) <= pfifo_wa_q3(7) and pfifo_we_i(2);
    if(pfifo_write(2) = '1')then
      pfifo_next_header(2) <= trailer;
    end if;
    pfifo_header(2) <= pfifo_next_header(2) and pfifo_write(2);
-- pfifo almost full
    pfifo_full(2) <= (ch2_pfifo_wc(5) and ch2_pfifo_wc(4)) and not GC_IFIFO;
-- pfifo almost empty and sram not empty which is checked by en_wr2sram
    if(pfifo_full(2) = '1')then
      pfifo_low(2) <= '0';
    elsif(ch2_pfifo_wc(5 downto 4) = "00")then
      pfifo_low(2) <= '1';
    end if;
-- set request when pfifo enters into low, reset request when sram empty or pfifo almost full
    if(pfifo_low(2) = '1' and (ch_wc_low(2) = '0' or ch2_wc(2) = '1' or (ch2_wc(1) = '1' and (ch2_wc(0) = '1' or sel_rp(2) = '0')) or
              ((pfifo_rqst_a(2) = '0' or ch_read_i(2) = '1') and ch2_wc(0) = '1' and sel_rp(2) = '0')))then
      pfifo_rqst_a(2) <= '1';
    else
      pfifo_rqst_a(2) <= '0';
    end if;
--
    en_wr2sram(2) <= pfifo_full(2) or not ch_empty(2);
    if(ch_read_i(2) = '1')then
      ch2_wp <= ch2_wp + 1;
    end if;
    if(sel_rp(2) = '1' or wr2pfifo(2) = '1')then
      ch2_rp <= ch2_rp + 1;
    end if;
  end if;
end process;
-- channel word count in sram
ec_ch_wc <= (en_wr2sram and ch_read_i) or sel_rp;
i_ch0_wc: lpm_counter
   port map(
           clock => clk,
           clk_en => ec_ch_wc(0),
		   updown => ch_read_i(0),
           aclr => cc_rst(0),
           q => ch0_wc);
i_ch1_wc: lpm_counter
   port map(
           clock => clk,
           clk_en => ec_ch_wc(1),
		   updown => ch_read_i(1),
           aclr => cc_rst(1),
           q => ch1_wc);
i_ch2_wc: lpm_counter
   port map(
           clock => clk,
           clk_en => ec_ch_wc(2),
		   updown => ch_read_i(2),
           aclr => cc_rst(2),
           q => ch2_wc);
process(clk,cc_rst)
begin
  if(cc_rst(0) = '1')then
    CS_OVER(0) <= '0';
    CS_FULL(0) <= '0';
  elsif(clk'event and clk = '1')then
    if((ch0_wc(15 downto 13)) >= '1' & GC_OVRL)then
      CS_OVER(0) <= '1';
    else
      CS_OVER(0) <= '0';
    end if;
    if(ch0_wc(15 downto 2) = "11111111111111")then
      CS_FULL(0) <= '1';
    else
      CS_FULL(0) <= '0';
    end if;
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(1) = '1')then
    CS_OVER(1) <= '0';
    CS_FULL(1) <= '0';
  elsif(clk'event and clk = '1')then
    if(ch1_wc(15 downto 13) >= '1' & GC_OVRL)then
      CS_OVER(1) <= '1';
    else
      CS_OVER(1) <= '0';
    end if;
    if(ch1_wc(15 downto 2) = "11111111111111")then
      CS_FULL(1) <= '1';
    else
      CS_FULL(1) <= '0';
    end if;
  end if;
end process;
process(clk,cc_rst)
begin
  if(cc_rst(2) = '1')then
    CS_OVER(2) <= '0';
    CS_FULL(2) <= '0';
  elsif(clk'event and clk = '1')then
    if(ch2_wc(15 downto 13) >= '1' & GC_OVRL)then
      CS_OVER(2) <= '1';
    else
      CS_OVER(2) <= '0';
    end if;
    if(ch2_wc(15 downto 2) = "11111111111111")then
      CS_FULL(2) <= '1';
    else
      CS_FULL(2) <= '0';
    end if;
  end if;
end process;
-- sram q output tri-state drivers
  sq(35 downto 33) <= "000" when aaa(0) = '0' else "ZZZ";
  sq(32 downto 31) <= sqr(32 downto 31) when aaa(1) = '0' else "ZZ";
  sq(30) <= sqr(30) when aaa(2) = '0' else 'Z';
  sq(29) <= sqr(29) when aaa(1) = '0' else 'Z';
  sq(28 downto 27) <= sqr(28 downto 27) when aaa(2) = '0' else "ZZ";
  sq(26 downto 25) <= sqr(26 downto 25) when aaa(0) = '0' else "ZZ";
  sq(24 downto 22) <= sqr(24 downto 22) when aaa(1) = '0' else "ZZZ";
  sq(21) <= sqr(21) when aaa(2) = '0' else 'Z';
  sq(20) <= sqr(20) when aaa(1) = '0' else 'Z';
  sq(19) <= sqr(19) when aaa(2) = '0' else 'Z';
  sq(18 downto 15) <= sqr(18 downto 15) when aaa(3) = '0' else "ZZZZ";
  sq(14) <= sqr(14) when aaa(4) = '0' else 'Z';
  sq(13) <= sqr(13) when aaa(3) = '0' else 'Z';
  sq(12) <= sqr(12) when aaa(4) = '0' else 'Z';
  sq(11) <= sqr(11) when aaa(5) = '0' else 'Z';
  sq(10) <= sqr(10) when aaa(4) = '0' else 'Z';
  sq(9) <= sqr(9) when aaa(5) = '0' else 'Z';
  sq(8) <= sqr(8) when aaa(2) = '0' else 'Z';
  sq(7 downto 6) <= sqr(7 downto 6) when aaa(3) = '0' else "ZZ";
  sq(5 downto 2) <= sqr(5 downto 2) when aaa(4) = '0' else "ZZZZ";
  sq(1 downto 0) <= sqr(1 downto 0) when aaa(5) = '0' else "ZZ";
end my_arch;
