library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity sim_ram is
    Port (
           clk133 : in std_logic;
           CC_RST : in std_logic_vector(2 downto 0);
-- lvds input signals
           ch_dav : in std_logic_vector(2 downto 0);
           ch0_data : in std_logic_vector(3 downto 0);
           ch1_data : in std_logic_vector(3 downto 0);
           ch2_data : in std_logic_vector(3 downto 0);
           ch_read : out std_logic_vector(2 downto 0);
-- strl and status
           GC_IFIFO : in std_logic;
           GC_RFIFO : in std_logic;
           GC_OVRL : in std_logic_vector(1 downto 0);
           CS_OVER: out std_logic_vector(2 downto 0);
           CS_FULL: out std_logic_vector(2 downto 0);
-- dummy
           adr_out: out std_logic_vector(4 downto 0);
-- pci fifo signals
           pfifo_read : in std_logic_vector(2 downto 0);
           pfifo_ch : in std_logic_vector(1 downto 0);
           CS_EMPTY: out std_logic_vector(2 downto 0);
           pfifo_we : out std_logic;
           pfifo_wa : out std_logic_vector(7 downto 0);
           pfifo_data : out std_logic_vector(3 downto 0)
         );
end sim_ram;

architecture my_arch of sim_ram is
COMPONENT ram
    generic (sram_width : integer);
	PORT(
		clk : IN std_logic;
		CC_RST : IN std_logic_vector(2 downto 0);
		ch_dav : IN std_logic_vector(2 downto 0);
		ch0_data : IN std_logic_vector(3 downto 0);
		ch1_data : IN std_logic_vector(3 downto 0);
		ch2_data : IN std_logic_vector(3 downto 0);
		GC_IFIFO : IN std_logic;
		GC_RFIFO : IN std_logic;
		GC_OVRL : IN std_logic_vector(1 downto 0);
		pfifo_read : IN std_logic_vector(2 downto 0);
		pfifo_ch : IN std_logic_vector(1 downto 0);          
		ch_read : OUT std_logic_vector(2 downto 0);
		nWR : OUT std_logic;
		RCS : OUT std_logic;
		adr : OUT std_logic_vector(18 downto 0);
		sd : IN std_logic_vector(3 downto 0);
		sq : OUT std_logic_vector(3 downto 0);
		CS_OVER : OUT std_logic_vector(2 downto 0);
		CS_FULL : OUT std_logic_vector(2 downto 0);
		CS_EMPTY : OUT std_logic_vector(2 downto 0);
		pfifo_we : OUT std_logic;
		pfifo_wa : OUT std_logic_vector(7 downto 0);
		pfifo_data : OUT std_logic_vector(3 downto 0)
		);
END COMPONENT;
COMPONENT lpm_ram_dp
   GENERIC ( LPM_WIDTH: POSITIVE := 4;
             LPM_WIDTHAD: POSITIVE := 14;
             LPM_NUMWORDS: NATURAL := 12288;
             LPM_TYPE: STRING := "LPM_RAM_DP";
             LPM_INDATA: STRING := "REGISTERED";
             LPM_OUTDATA: STRING := "REGISTERED";
             LPM_RDADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_WRADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_FILE: STRING := "UNUSED";
             LPM_HINT: STRING := "UNUSED"

           );
   PORT    ( rdaddress, wraddress: IN STD_LOGIC_VECTOR(LPM_WIDTHAD-1 DOWNTO 0);
             rdclock, wrclock: IN STD_LOGIC := '0';
             rden, rdclken, wrclken: IN STD_LOGIC := '1';
             wren: IN STD_LOGIC; 
             data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0);
             q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
signal nWR: std_logic;
signal nWR_dl: std_logic;
signal rd_fifo: std_logic;
signal RCS: std_logic;
signal wr_mem: std_logic;
signal mem_adr: std_logic_vector(13 downto 0);
signal adr: std_logic_vector(18 downto 0);
signal sd: std_logic_vector(3 downto 0);
signal sq: std_logic_vector(3 downto 0);
signal fifo_do: std_logic_vector(3 downto 0);
begin
sram: ram
    generic map(sram_width => 4) 
    PORT MAP(
		clk => clk133,
		CC_RST => cc_rst,
		ch_dav => ch_dav,
		ch0_data => ch0_data,
		ch1_data => ch1_data,
		ch2_data => ch2_data,
		ch_read => ch_read,
		nWR => nWR,
		RCS => RCS,
		adr => ADR,
		sd => SD,
		sq => SQ,
		GC_IFIFO => gc_ififo,
		GC_RFIFO => gc_rfifo,
		GC_OVRL => GC_OVRL,
		CS_OVER => cs_over,
		CS_FULL => cs_full,
		pfifo_read => pfifo_read,
		pfifo_ch => pfifo_ch,
		CS_EMPTY => CS_EMPTY,
		pfifo_we => pfifo_we,
		pfifo_wa => pfifo_wa,
		pfifo_data => pfifo_data
	);
i_pci_fifo: lpm_ram_dp
   port map(
           rdaddress => mem_adr,
           wraddress => mem_adr,
           wrclock => clk133,
           rdclock => clk133,
           wren => wr_mem,
           data => sq,
           q => fifo_do);
process(clk133)
begin
  if(clk133'event and clk133 = '1')then
    nWR_dl <= nWR;
    rd_fifo <= nWR_dl;
  end if;
end process;
sd <= fifo_do when rd_fifo = '1' else sq;
wr_mem <= rcs and not nWR;
mem_adr <= adr(17 downto 16) & adr(11 downto 0);
adr_out <= adr(18) & adr(15 downto 12);
end my_arch;
